# -*- coding: utf-8 -*-

"""
/***************************************************************************
 illini_drainage_tools
                                 A QGIS plugin
 Performs Specific Draiange Related Tasks and Analysis on a Site
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2022-03-15
        copyright            : (C) 2022 by FALASY  Anamelechi
        email                : fvw.services@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

__author__ = 'FALASY  Anamelechi'
__date__ = '2022-03-15'
__copyright__ = '(C) 2022 by FALASY  Anamelechi'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

import os, math
import inspect
from qgis.PyQt.QtGui import QIcon

from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterRasterLayer
from qgis.core import QgsProcessingParameterFeatureSource
from qgis.core import QgsProcessingParameterFeatureSink
from qgis.core import QgsProcessingParameterBoolean
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterNumber
from qgis.core import QgsProcessingParameterField

import processing

from PyQt5 import QtWidgets
from qgis.PyQt.QtCore import QCoreApplication, QVariant

from qgis.core import *
from collections import Counter
import time
import numpy as np

class MultipleMainsFixAlgorithm(QgsProcessingAlgorithm):
    
    def tr(self, string):
        return QCoreApplication.translate('Processing', string)
        
    def createInstance(self):
        return MultipleMainsFixAlgorithm()
                
    def name(self):
        return '5. Multiple-Main Node Generator'

    def displayName(self):
        return self.tr(self.name())

    def group(self):
        return self.tr(self.groupId())

    def groupId(self):
        return ''
        
    def icon(self):
        cmd_folder = os.path.split(inspect.getfile(inspect.currentframe()))[0]
        icon = QIcon(os.path.join(os.path.join(cmd_folder, 'logo.png')))
        return icon
        
    def shortHelpString(self):
        return self.tr( """This tool is used to clean up a line layer from its global properties in space. 
        
        Workflow:         
        1. Choose a Vector Line layer of Multiple Mains 
        2. Click to select the Field ID that represents the line segments from the displayed line layer
        3. Choose a desired Coordinate Reference System for displaying the generated points         
        4. Click on \"Run\"               
                
        The script will give out an output. 
        Use this output with "Routine 6" to have a topologically sound tile network. 
                
        The help link in the Graphical User Interface (GUI) provides more information about the plugin.
        """)   
        
    def helpUrl(self):
        return "http://www.wq.illinois.edu/DG/DrainageGuide.html" 
    
    
    def initAlgorithm(self, config=None):
        
        self.addParameter(QgsProcessingParameterVectorLayer('VectorLineLayer', 'Proposed Tile Lines', types=[QgsProcessing.TypeVectorLine], defaultValue=None))
        self.addParameter(QgsProcessingParameterField('FGH', 'Field ID from Line Layer', parentLayerParameterName = 'VectorLineLayer', type = QgsProcessingParameterField.Any)) 
        self.addParameter(QgsProcessingParameterCrs('CRS', 'Coordinate Reference System', defaultValue='EPSG:3435'))
        self.addParameter(QgsProcessingParameterFeatureSink('LineFixes', 'New Line Nodes', type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=True, supportsAppend=True, defaultValue=None))       
                               
    def processAlgorithm(self, parameters, context, model_feedback):
        # Use a multi-step feedback, so that individual child algorithm progress reports are adjusted for the
        # overall progress through the model
        
        feedback = QgsProcessingMultiStepFeedback(6, model_feedback)
        results = {}
        outputs = {}
                
        alg_params = {'INPUT': parameters['VectorLineLayer'], 'START_DISTANCE':0, 'END_DISTANCE':5,'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}

        # Check if vector line layer 'VectorLineLayer' is in geogrephic coordinates
        vector_layer = self.parameterAsVectorLayer(parameters, 'VectorLineLayer', context)
        if vector_layer.crs().isGeographic():

            w = QtWidgets.QWidget()
            b = QtWidgets.QLabel(w)
            w.setGeometry(400,400,800,20)
            w.setWindowTitle("Attention: vector line layers in geographic coordinates are not allowed! Ending plugin without slope calculation...")
            w.show()
            time.sleep(10)
            return results

        # Extend Lines     
        outputs['ExtendLines'] = processing.run('native:extendlines', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #1
        
        feedback.setCurrentStep(1)
        if feedback.isCanceled():
            return {}
        
        # MultiPart To SingleParts
        alg_params = {'INPUT': outputs['ExtendLines']['OUTPUT'], 'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
        
        outputs['MultiPartToSingleParts'] = processing.run('native:multiparttosingleparts', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #2

        feedback.setCurrentStep(2)
        if feedback.isCanceled():
            return {}

        # Dissolve Line Split lines by maximum length
        alg_params = {'INPUT': outputs['MultiPartToSingleParts']['OUTPUT'], 'FIELD': parameters['FGH'], 'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}        

        outputs['Dissolve'] = processing.run('native:dissolve', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #3

        feedback.setCurrentStep(3)
        if feedback.isCanceled():
            return {}
        
        # Extract Vertices
        alg_params = {'INPUT': outputs['Dissolve']['OUTPUT'], 'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
        
        outputs['ExtractVertices'] = processing.run('native:extractvertices', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #4

        feedback.setCurrentStep(4)
        if feedback.isCanceled():
            return {}

        # Add X/Y Fields
        alg_params = {'INPUT': outputs['ExtractVertices']['OUTPUT'], 'CRS': parameters['CRS'], 'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
        
        outputs['AddxyFields'] = processing.run('native:addxyfields', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #5

        feedback.setCurrentStep(5)
        if feedback.isCanceled():
            return {}

        # Snap Geometries to Layers        
        alg_params = {'INPUT': outputs['AddxyFields']['OUTPUT'], 'REFERENCE_LAYER': outputs['AddxyFields']['OUTPUT'], 'TOLERANCE': 10, 'BEHAVIOR': 0, 'OUTPUT': parameters['LineFixes']}
                
        outputs['SnapGeometries'] = processing.run('native:snapgeometries', alg_params, context=context, feedback=feedback, is_child_algorithm=True) #6
                
        results['LineFixes'] = outputs['SnapGeometries']['OUTPUT']
        return results      
    
